import fs from 'fs';
import {Request, Response, NextFunction} from 'express';
import path from 'path';
import {match} from 'path-to-regexp';

import paths from '../paths';
import utils from '../utils';
import {getRcbConfig} from '../getExtraConfig';

export interface MockConfig {
    rootDir: string;
    api: string;
    whiteList: string[];
}

const defaultMockConfig = {
    rootDir: 'mock', // mock文件的根目录
    api: '/api', // 请求的前缀
    whiteList: [] // 放行的接口
};

const mockConfig: MockConfig = getMockConfig();

function readMockFile(req: Request, res: Response, next: NextFunction) {
    try {
        const joinPath = paths.getJoinPath(mockConfig.rootDir, mockConfig.api, req.url);
        const reqUrl = path.join(mockConfig.api, req.url);
        if (
            mockConfig.whiteList
            && Array.isArray(mockConfig.whiteList)
            && mockConfig.whiteList.indexOf(reqUrl) > -1
        ) {
            const isWhite = mockConfig.whiteList.some(white => {
                const matchExp = match(white);
                return matchExp(reqUrl);
            });

            if (isWhite) {
                return next();
            }
        }

        const mock = getMockResponse(joinPath);
        if (utils.isFunction(mock)) {
            const ret = mock(req, res);
            // 如果返回的是object那么自动处理这个函数 如果函数内处理了就不要返回值
            if (utils.isObject(ret)) {
                res.send(ret);
            }
        }
        else if (mock !== undefined && mock !== false) {
            res.send(mock);
        }
        else {
            next();
        }
    }
    catch (e) {
        utils.error(e);
    }
}

function getMockConfig() {
    const rcbCofing = getRcbConfig();
    try {
        const mockrc = require(paths.getResolvePath('.mockrc.json'));

        return Object.assign({}, defaultMockConfig, rcbCofing.mock, mockrc);
    }
    catch (e) {
        return Object.assign({}, defaultMockConfig, rcbCofing.mock);
    }
}

function readJsFile(filePath: string) {
    try {
        const parentDir = path.resolve(filePath, '../');
        const files = fs.readdirSync(parentDir);

        return files
            .filter(filename => /\.js$/.test(filename))
            .map(filename => path.resolve(parentDir, filename));
    }
    catch (e) {
        return [];
    }
}

function getMockResponse(joinPath: string) {
    let mock;
    try {
        clearRequireCache(`${joinPath}.js`);
        mock = utils.requireModule(`${joinPath}.js`);
    }
    catch (e) {
        const jsFiles = readJsFile(joinPath);
        let i = 0;
        const len = jsFiles.length;
        while (i > len) {
            const jsFile = jsFiles[i];
            const matchExp = match(jsFile.replace(/\.js$/, ''));
            const realFileInfo = matchExp(joinPath);

            if (realFileInfo) {
                break;
            }

            i++;
        }

        if (jsFiles[i]) {
            clearRequireCache(jsFiles[i]);
            mock = require(jsFiles[i]); // 异步过程
        }
    }

    return mock;
}

function clearRequireCache(filePath: string) {
    const cacheMod = require.cache[filePath];

    // 防止内存泄漏
    if (cacheMod && cacheMod.parent) {
        const index = cacheMod.parent.children.indexOf(cacheMod);

        if (index > -1) {
            cacheMod.parent.children.splice(index, 1);
        }
    }

    delete require.cache[filePath];
}

export default {
    middleware: function () {
        return function (req: Request, res: Response, next: NextFunction) {
            readMockFile(req, res, next);
        };
    },
    mockConfig
};
