import './babelRegister';

import webpack from 'webpack';
import {Application} from 'express';
import {Options} from './start';
import WebpackDevServer from 'webpack-dev-server';

import mockMiddleware from './middleware/mock';
import loggerMiddleware from './middleware/logger';

import utils from './utils';
import clearConsole from './clearConsole';
import getExtraConfig from './getExtraConfig';
import getWebpackConfig from './getWebpackConfig';
import compilerErrorHandle from './compilerErrorHandle';

const HOST = '127.0.0.1';
const PORT = 3000;

export default function runServer(options: Options) {
    const extraConfig = getExtraConfig();
    
    const devServerConfig: WebpackDevServer.Configuration = {
        host: extraConfig.rcb.host || HOST,
        port: extraConfig.rcb.port || PORT,
        proxy: extraConfig.rcb.proxy,
        quiet: true,
        overlay: true,
        disableHostCheck: true,
        historyApiFallback: {
            htmlAcceptHeaders: ['text/html', 'application/xhtml+xml', 'application/xml']
        },
        compress: true,
        clientLogLevel: 'warn',
        hot: true,
        inline: true,
        watchOptions: {
            ignored: /node_modules/
        },
        open: true,
        setup(app: Application) {
            if (options.mock) {
                // mock代理的url不会在node中log出来，只有proxy代理的url才会log方便查看是哪里出了问题
                app.use(mockMiddleware.mockConfig.api, mockMiddleware.middleware());
            }
            app.use(loggerMiddleware());
        }
    };
    const webpackConfig: webpack.Configuration = getWebpackConfig(extraConfig.webpack, {devServer: devServerConfig});

    const URL = `http://${devServerConfig.host}:${devServerConfig.port}`;
    const compiler = webpack(webpackConfig);
    let server = new WebpackDevServer(compiler, devServerConfig);
    let isFirstCompiler = true;

    compiler.hooks.beforeCompile.tap('beforeCompile', function () {
        if (isFirstCompiler) {
            return;
        }

        clearConsole();
        utils.success(utils.chalk.green('Rebuild start...'));
    });

    compiler.hooks.done.tap('dev server', function (stats) {
        if (isFirstCompiler) {
            const proxyPaths = utils.getWebpackProxyPath(webpackConfig);

            utils.success(`${utils.chalk.green('Local:')}  ${URL}`);
            if (proxyPaths.length > 0) {
                utils.success(`${utils.chalk.green('Proxy:')}  [${proxyPaths.join(', ')}]`);
            }
            utils.success(utils.chalk.green('Compiled successfully!'));

            isFirstCompiler = false;
        }
        else {
            utils.success(utils.chalk.green('Compiled successfully!'));
        }

        compilerErrorHandle(stats, null, isFirstCompiler);
    });

    if (!devServerConfig.port || !devServerConfig.host) {
        utils.error(`port: ${devServerConfig.port}, host: ${devServerConfig.host}`);
        process.exit(0);
    }

    server.listen(devServerConfig.port, devServerConfig.host, function () {
        if (isFirstCompiler) {
            clearConsole();
        }
        utils.success(utils.chalk.green('Server starting...'));
    });
};
